package ru.ulstu.fc.rule.controller;

import io.swagger.v3.oas.annotations.Hidden;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import ru.ulstu.fc.rule.model.Antecedent;
import ru.ulstu.fc.rule.model.InferenceForm;
import ru.ulstu.fc.rule.service.FuzzyInferenceService;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

@Controller
@Hidden
public class InferenceMvcController {
    private final FuzzyInferenceService fuzzyInferenceService;

    public InferenceMvcController(FuzzyInferenceService fuzzyInferenceService) {
        this.fuzzyInferenceService = fuzzyInferenceService;
    }

    @GetMapping("/")
    public String initInference(Model model) {
        model.addAttribute("ageAntecedents", getAgeAntecedents());
        model.addAttribute("incomeAntecedents", getIncomeAntecedents());
        model.addAttribute("inferenceForm", new InferenceForm());
        return "index";
    }

    @RequestMapping(value = "get-inference", method = RequestMethod.POST)
    public String getInference(@ModelAttribute InferenceForm inferenceForm, Model model) {
        model.addAttribute("ageAntecedents", getAgeAntecedents());
        model.addAttribute("incomeAntecedents", getIncomeAntecedents());
        model.addAttribute("inferenceForm", inferenceForm);
        model.addAttribute("response", fuzzyInferenceService.getFuzzyInference(
                Map.of("возраст", Double.valueOf(inferenceForm.getAgeAntecedent()),
                        "доход", Double.valueOf(inferenceForm.getIncomeAntecedent())
                )));
        return "index";
    }

    private List<Antecedent> getAgeAntecedents() {
        return Arrays.asList(
                new Antecedent("молодой", "30"),
                new Antecedent("средний", "45"),
                new Antecedent("старый", "60"));
    }

    private List<Antecedent> getIncomeAntecedents() {
        return Arrays.asList(
                new Antecedent("небольшой", "20000"),
                new Antecedent("средний", "90000"),
                new Antecedent("высокий", "200000"));
    }
}
